; comparator_oscillator_jan11.a51   comparators watch charging cap, make oscillation;  
;  assumes photodiode (or, simpler, variable R) between V+ and cap, monitored by compar + input
; Jan. 11 re-assign MOSFET gate to P0.3, and let Wizard handle configuration


; rev path 3/16
	$NOSYMBOLS	; keeps listing short
	$INCLUDE (C:\SiLabs\MCU\INC\c8051f410.inc) 

	STACKBOT EQU 80h

; PORT USE:
; comparator inputs: - is P0.6, + is P0.7
; discharge MOSFET: P0.3

	ORG 0h
	LJMP STARTUP

	ORG 080h
				
STARTUP: MOV SP, #STACKBOT-1

	ACALL USUAL_SETUP
	ACALL Init_Device
	CLR P0.3			; MOSFET off
	SETB P0.6			; write One's to pins to be used as inputs
	SETB P0.7

OSC_LOOP:	MOV A, CPT0CN		; get this where we can test rising-edge flag
  JNB ACC.6, OSC_LOOP		; branch on comparator output (not flag)
	SETB P0.3					; discharge cap (turn on MOSFET)
	ACALL delay				; ...allow enough time for full discharge of cap
	CLR P0.3				; ...turn off MOSFET
	SJMP OSC_LOOP


;------- INITIALIZATIONS
USUAL_SETUP: ANL   PCA0MD, #NOT(040h)        ; Disable the WDT.
                                             ; Clear Watchdog Enable bit
	; Configure the Oscillator
             ORL   OSCICN, #04h         ; sysclk = 24.5 Mhz / 8
						RET
		
DELAY: PUSH ACC
			MOV A, #80h				; works pretty well: 10h caused step up on release of reset
			DJNZ ACC,$
			POP ACC
			RET

; Called from the Init_Device label
Comparator_Init:
    mov  CPT0CN,    #085h    ; enable comparator 0; output high if + > - ; hysteresis +- 5 mV

    mov  CPT0MX,    #033h    ; establish input pins for comparators (P0.6, P0.)
    mov  CPT0MD,    #003h    ; lowest power, lowest speed setting for comparator
    ret

Port_IO_Init:
    ; P0.0  -  Skipped,     Open-Drain, Digital
    ; P0.1  -  Skipped,     Open-Drain, Digital
    ; P0.2  -  Skipped,     Open-Drain, Digital
    ; P0.3  -  Skipped,     Push-Pull,  Digital
    ; P0.4  -  Skipped,     Open-Drain, Digital
    ; P0.5  -  Skipped,     Open-Drain, Digital
    ; P0.6  -  CP0 (Cmp0),  Open-Drain, Analog
    ; P0.7  -  CP0A (Cmp0), Open-Drain, Analog

    mov  P0MDIN,    #03Fh    ; make the two comparator inputs analog (P0.6, P0.7)
    mov  P0MDOUT,   #008h    ; make P0.3 push-pull, to drive MOSFET gate
    mov  P0SKIP,    #03Fh    ; skip portpins so as to assign comparator inputs to P0.6, P0.7
;    mov  XBR0,      #030h	   ; route comparator output to portpin, route CP0 to port pin
    mov  XBR1,      #040h    ; enable crossbar
    ret

; Initialization function for device,
; Call Init_Device from your main program
Init_Device:
    lcall Comparator_Init
    lcall Port_IO_Init
    ret


END
	
